@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23838 $
- * $Author: csoutheren $
- * $Date: 2009-12-04 01:52:56 -0600 (Fri, 04 Dec 2009) $
+ * $Revision: 27097 $
+ * $Author: rjongbloed $
+ * $Date: 2012-03-02 19:29:34 -0600 (Fri, 02 Mar 2012) $
  */
 
 #ifndef PTLIB_PXML_H
@@ -84,13 +84,13 @@ class PXMLBase : public PObject
       IsStandAlone
     };
 
-    PXMLBase(Options opts = NoOptions)
+    PXMLBase(int opts = NoOptions)
       : m_options(opts) { }
 
-    void SetOptions(Options opts)
+    void SetOptions(int opts)
       { m_options = opts; }
 
-    Options GetOptions() const { return m_options; }
+    int GetOptions() const { return m_options; }
 
     virtual PBoolean IsNoIndentElement(
       const PString & /*elementName*/
@@ -100,22 +100,21 @@ class PXMLBase : public PObject
     }
 
   protected:
-    Options m_options;
+    int m_options;
 };
 
 
 class PXML : public PXMLBase
 {
-  PCLASSINFO(PXML, PObject);
+    PCLASSINFO(PXML, PObject);
   public:
-
     PXML(
-      Options options = NoOptions,
+      int options = NoOptions,
       const char * noIndentElements = NULL
     );
     PXML(
       const PString & data,
-      Options options = NoOptions,
+      int options = NoOptions,
       const char * noIndentElements = NULL
     );
 
@@ -127,23 +126,6 @@ class PXML : public PXMLBase
     bool IsDirty() const;
 
     bool Load(const PString & data, Options options = NoOptions);
-
-#if P_HTTP
-    bool StartAutoReloadURL(
-      const PURL & url, 
-      const PTimeInterval & timeout, 
-      const PTimeInterval & refreshTime,
-      Options options = NoOptions
-    );
-    bool StopAutoReloadURL();
-    PString GetAutoReloadStatus() { PWaitAndSignal m(autoLoadMutex); PString str = autoLoadError; return str; }
-    bool AutoLoadURL();
-    virtual void OnAutoLoad(PBoolean ok);
-
-    bool LoadURL(const PURL & url);
-    bool LoadURL(const PURL & url, const PTimeInterval & timeout, Options options = NoOptions);
-#endif // P_HTTP
-
     bool LoadFile(const PFilePath & fn, Options options = NoOptions);
 
     virtual void OnLoaded() { }
@@ -193,7 +175,12 @@ class PXML : public PXMLBase
       OptionalAttributeWithValueMatching,
       OptionalElementWithBodyMatching,
       SetDefaultNamespace,
-      SetNamespace
+      SetNamespace,
+
+      RequiredAttributeWithValueMatchingEx = RequiredAttributeWithValueMatching + 0x8000,
+      OptionalAttributeWithValueMatchingEx = OptionalAttributeWithValueMatching + 0x8000,
+      RequiredElementWithBodyMatchingEx    = RequiredElementWithBodyMatching    + 0x8000,
+      OptionalElementWithBodyMatchingEx    = OptionalElementWithBodyMatching    + 0x8000
     };
 
     struct ValidationContext {
@@ -219,6 +206,7 @@ class PXML : public PXMLBase
     bool Validate(const ValidationInfo * validator);
     bool ValidateElements(ValidationContext & context, PXMLElement * baseElement, const ValidationInfo * elements);
     bool ValidateElement(ValidationContext & context, PXMLElement * element, const ValidationInfo * elements);
+    bool LoadAndValidate(const PString & body, const PXML::ValidationInfo * validator, PString & error, int options = NoOptions);
 
     PString  GetErrorString() const { return m_errorString; }
     unsigned GetErrorColumn() const { return m_errorColumn; }
@@ -229,11 +217,6 @@ class PXML : public PXMLBase
 
     PMutex & GetMutex() { return rootMutex; }
 
-#if P_HTTP
-    PDECLARE_NOTIFIER(PTimer,  PXML, AutoReloadTimeout);
-    PDECLARE_NOTIFIER(PThread, PXML, AutoReloadThread);
-#endif // P_HTTP
-
     // static methods to create XML tags
     static PString CreateStartTag (const PString & text);
     static PString CreateEndTag (const PString & text);
@@ -243,7 +226,7 @@ class PXML : public PXMLBase
     static PString EscapeSpecialChars(const PString & string);
 
   protected:
-    void Construct(Options options, const char * noIndentElements);
+    void Construct(int options, const char * noIndentElements);
     PXMLElement * rootElement;
     PMutex rootMutex;
 
@@ -252,14 +235,6 @@ class PXML : public PXMLBase
     PString version, encoding;
     StandAloneType m_standAlone;
 
-#if P_HTTP
-    PTimer autoLoadTimer;
-    PURL autoloadURL;
-    PTimeInterval autoLoadWaitTime;
-    PMutex autoLoadMutex;
-    PString autoLoadError;
-#endif // P_HTTP
-
     PStringStream m_errorString;
     unsigned      m_errorLine;
     unsigned      m_errorColumn;
@@ -270,6 +245,43 @@ class PXML : public PXMLBase
     PString m_defaultNameSpace;
 };
 
+
+#if P_HTTP
+class PXML_HTTP : public PXML
+{
+    PCLASSINFO(PXML_HTTP, PXML);
+  public:
+    PXML_HTTP(
+      int options = NoOptions,
+      const char * noIndentElements = NULL
+    );
+
+    bool StartAutoReloadURL(
+      const PURL & url, 
+      const PTimeInterval & timeout, 
+      const PTimeInterval & refreshTime,
+      Options options = NoOptions
+    );
+    bool StopAutoReloadURL();
+    PString GetAutoReloadStatus() { PWaitAndSignal m(autoLoadMutex); PString str = autoLoadError; return str; }
+    bool AutoLoadURL();
+    virtual void OnAutoLoad(PBoolean ok);
+
+    bool LoadURL(const PURL & url);
+    bool LoadURL(const PURL & url, const PTimeInterval & timeout, Options options = NoOptions);
+
+  protected:
+    PDECLARE_NOTIFIER(PTimer,  PXML_HTTP, AutoReloadTimeout);
+    PDECLARE_NOTIFIER(PThread, PXML_HTTP, AutoReloadThread);
+
+    PTimer autoLoadTimer;
+    PURL autoloadURL;
+    PTimeInterval autoLoadWaitTime;
+    PMutex autoLoadMutex;
+    PString autoLoadError;
+};
+#endif // P_HTTP
+
 ////////////////////////////////////////////////////////////
 
 PARRAY(PXMLObjectArray, PXMLObject);
@@ -352,7 +364,7 @@ class PXMLElement : public PXMLObject {
     PCaselessString GetPathName() const;
 
     void SetName(const PString & v)
-      { name = v; }
+    { name = v; }
 
     PINDEX GetSize() const
       { return subObjects.GetSize(); }
@@ -452,7 +464,7 @@ class PXMLParser : public PXMLBase
 {
   PCLASSINFO(PXMLParser, PXMLBase);
   public:
-    PXMLParser(Options options = NoOptions);
+    PXMLParser(int options = NoOptions);
     ~PXMLParser();
     bool Parse(const char * data, int dataLen, bool final);
     void GetErrorInfo(PString & errorString, unsigned & errorCol, unsigned & errorLine);
