@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22722 $
+ * $Revision: 26456 $
  * $Author: rjongbloed $
- * $Date: 2009-05-26 06:59:58 -0500 (Tue, 26 May 2009) $
+ * $Date: 2011-09-22 00:27:10 -0500 (Thu, 22 Sep 2011) $
  */
 
 #include "precompile.h"
@@ -44,6 +44,7 @@ VidTest::VidTest()
   : PProcess("PwLib Video Example", "vidtest", 1, 0, ReleaseCode, 0)
   , m_grabber(NULL)
   , m_display(NULL)
+  , m_secondary(NULL)
 {
 }
 
@@ -160,8 +161,8 @@ void VidTest::Main()
       format = PVideoDevice::Auto;
     else {
       cerr << "Illegal video input format name \"" << formatString << '"' << endl;
-    return;
-  }
+      return;
+    }
     if (!m_grabber->SetVideoFormat(format)) {
       cerr << "Video input device could not be set to format \"" << formatString << '"' << endl;
       return;
@@ -321,6 +322,31 @@ void VidTest::Main()
       if (cin.eof() || cmd == "q" || cmd == "x" || cmd == "quit" || cmd == "exit")
         break;
 
+      if (cmd == "stop") {
+        if (!m_grabber->Stop())
+          cout << "\nCould not stop video input device" << endl;
+        continue;
+      }
+
+      if (cmd == "start") {
+        if (!m_grabber->Start())
+          cout << "\nCould not stop video input device" << endl;
+        continue;
+      }
+
+      if (cmd == "secondary") {
+        if (m_secondary != NULL) {
+          delete m_secondary;
+          m_secondary = NULL;
+        }
+        else {
+          m_secondary = PVideoOutputDevice::CreateOpenedDevice(PString::Empty(), m_display->GetDeviceName());
+          if (m_secondary == NULL)
+            cout << "\nCould not start secondary video output device" << endl;
+        }
+        continue;
+      }
+
       if (cmd == "fg") {
         if (!m_grabber->SetVFlipState(!m_grabber->GetVFlipState()))
           cout << "\nCould not toggle Vflip state of video input device" << endl;
@@ -345,6 +371,8 @@ void VidTest::Main()
       }
 
       cout << "Select:\n"
+              "  stop   : Stop grabbing\n"
+              "  start  : Start grabbing\n"
               "  fg     : Flip video input top to bottom\n"
               "  fd     : Flip video output top to bottom\n"
               "  qcif   : Set size of grab & display to qcif\n"
@@ -357,6 +385,7 @@ void VidTest::Main()
   cout << "Exiting." << endl;
   m_exitGrabAndDisplay.Signal();
 
+  delete m_secondary;
   delete m_display;
   delete m_grabber;
 }
@@ -368,6 +397,7 @@ void VidTest::GrabAndDisplay(PThread &, INT)
   unsigned frameCount = 0;
   bool oldGrabberState = true;
   bool oldDisplayState = true;
+  bool oldSecondaryState = true;
 
   if (!m_grabber->Start()) {
     cout << "Could not start video grabber!" << endl;
@@ -397,20 +427,28 @@ void VidTest::GrabAndDisplay(PThread &, INT)
       if (m_converters[frameIndex].Convert(frames[frameIndex],
                                             frames[frameIndex+1].GetPointer(m_converters[frameIndex].GetMaxDstFrameBytes())))
         m_converters[frameIndex].GetDstFrameSize(width, height);
-      else {
+      else
         cerr << "Frame conversion failed!" << endl;
-      }
     }
 
     m_display->SetFrameSize(width, height);
 
     bool displayState = m_display->SetFrameData(0, 0, width, height, frames.back());
-    if (oldDisplayState != displayState)
-    {
+    if (oldDisplayState != displayState) {
       oldDisplayState = displayState;
       cerr << "Frame display " << (displayState ? "restored." : "failed!") << endl;
     }
 
+    if (m_secondary != NULL) {
+      m_secondary->SetFrameSize(width, height);
+
+      displayState = m_secondary->SetFrameData(0, 0, width, height, frames.back());
+      if (oldSecondaryState != displayState) {
+        oldSecondaryState = displayState;
+        cerr << "Secondary Frame display " << (displayState ? "restored." : "failed!") << endl;
+      }
+    }
+
     if (m_pauseGrabAndDisplay.Wait(0)) {
       m_pauseGrabAndDisplay.Acknowledge();
       m_resumeGrabAndDisplay.Wait();
