@@ -23,9 +23,9 @@
  *
  * Copyright 2003 Equivalence Pty. Ltd.
  *
- * $Revision: 22297 $
+ * $Revision: 28359 $
  * $Author: rjongbloed $
- * $Date: 2009-03-25 19:37:59 -0500 (Wed, 25 Mar 2009) $
+ * $Date: 2012-09-14 18:44:41 -0500 (Fri, 14 Sep 2012) $
  */
 
 #ifdef __GNUC__
@@ -39,12 +39,60 @@
 
 #define new PNEW
 
+#define USE_RESOLVER_CACHING    1
+#define RESOLVER_CACHE_TIMEOUT  30000
 
 #if P_DNS
 
+#ifdef _WIN32
+  #pragma comment(lib, "DnsAPI.Lib")
+  #pragma message("DNS support enabled")
+#else
+  enum { DnsFreeFlat = 0 };
+#endif
+
 
 /////////////////////////////////////////////////
 
+#if defined(P_HAS_RESOLVER) ||  defined(USE_RESOLVER_CACHING)
+
+void PDnsRecordListFree(PDNS_RECORD rec, int /* FreeType */)
+{
+  while (rec != NULL) {
+    PDNS_RECORD next = rec->pNext;
+    free(rec);
+    rec = next;
+  }
+}
+
+#endif
+
+#if defined(USE_RESOLVER_CACHING) || (defined(P_HAS_RESOLVER) && !defined(P_HAS_RES_NINIT))
+
+static PMutex & GetDNSMutex()
+{
+  static PMutex mutex;
+  return mutex;
+}
+
+#endif
+
+#if defined(USE_RESOLVER_CACHING) 
+
+struct DNSCacheInfo {
+  DNSCacheInfo() : m_results(NULL), m_status(-1) { }
+  PTime         m_time;
+  PDNS_RECORD   m_results;
+  DNS_STATUS    m_status;
+};
+
+typedef std::map<std::string, DNSCacheInfo> DNSCache;
+
+static PTime g_lastAgeTime(0);
+static DNSCache g_dnsCache;
+
+#endif
+
 #ifdef P_HAS_RESOLVER
 
 static PBoolean GetDN(const BYTE * reply, const BYTE * replyEnd, BYTE * & cp, char * buff)
@@ -84,19 +132,19 @@ static PBoolean ProcessDNSRecords(
 
     // get the name
     char pName[MAXDNAME];
-    if (!GetDN(reply, replyEnd, cp, pName)) 
+    if (!GetDN(reply, replyEnd, cp, pName))
       return PFalse;
 
     // get other common parts of the record
     WORD  type;
-    WORD  dnsClass;
-    DWORD ttl;
+    //WORD  dnsClass;
+    //DWORD ttl;
     WORD  dlen;
 
-    GETSHORT(type,     cp);
-    GETSHORT(dnsClass, cp);
-    GETLONG (ttl,      cp);
-    GETSHORT(dlen,     cp);
+    GETSHORT(type, cp);
+    cp += 2; // GETSHORT(dnsClass, cp);
+    cp += 4; // GETLONG (ttl,      cp);
+    GETSHORT(dlen, cp);
 
     BYTE * data = cp;
     cp += dlen;
@@ -138,6 +186,15 @@ static PBoolean ProcessDNSRecords(
         GETLONG(newRecord->Data.A.IpAddress, data);
         break;
 
+      case T_AAAA:
+        newRecord = (PDNS_RECORD)malloc(sizeof(DnsRecord)); 
+        memset(newRecord, 0, sizeof(DnsRecord));
+        GETLONG(newRecord->Data.AAAA.Ip6Address[0], data);
+        GETLONG(newRecord->Data.AAAA.Ip6Address[1], data);
+        GETLONG(newRecord->Data.AAAA.Ip6Address[2], data);
+        GETLONG(newRecord->Data.AAAA.Ip6Address[3], data);
+        break;
+
       case T_NS:
         newRecord = (PDNS_RECORD)malloc(sizeof(DnsRecord)); 
         memset(newRecord, 0, sizeof(DnsRecord));
@@ -169,25 +226,6 @@ static PBoolean ProcessDNSRecords(
   return PTrue;
 }
 
-void DnsRecordListFree(PDNS_RECORD rec, int /* FreeType */)
-{
-  while (rec != NULL) {
-    PDNS_RECORD next = rec->pNext;
-    free(rec);
-    rec = next;
-  }
-}
-
-#if ! P_HAS_RES_NINIT
-
-static PMutex & GetDNSMutex()
-{
-  static PMutex mutex;
-  return mutex;
-}
-
-#endif
-
 DNS_STATUS DnsQuery_A(const char * service,
                               WORD requestType,
                              DWORD options,
@@ -256,7 +294,7 @@ DNS_STATUS DnsQuery_A(const char * service,
        ntohs(reply.hdr.nscount),
        ntohs(reply.hdr.arcount),
        results)) {
-    DnsRecordListFree(*results, 0);
+    PDnsRecordListFree(*results, 0);
     return -1;
   }
 
@@ -316,17 +354,21 @@ PDNS::SRVRecord * PDNS::SRVRecordList::HandleDNSRecord(PDNS_RECORD dnsRecord, PD
     record->priority = dnsRecord->Data.SRV.wPriority;
     record->weight   = dnsRecord->Data.SRV.wWeight;
 
-    // see if any A records match this hostname
+    // see if any A or AAAA records match this hostname
     PDNS_RECORD aRecord = results;
     while (aRecord != NULL) {
       if ((dnsRecord->Flags.S.Section == DnsSectionAdditional) && (dnsRecord->wType == DNS_TYPE_A)) {
         record->hostAddress = PIPSocket::Address(dnsRecord->Data.A.IpAddress);
         break;
       }
+      if ((dnsRecord->Flags.S.Section == DnsSectionAdditional) && (dnsRecord->wType == DNS_TYPE_AAAA)) {
+        record->hostAddress = PIPSocket::Address(16, (BYTE *)&dnsRecord->Data.AAAA.Ip6Address);
+        break;
+      }
       aRecord = aRecord->pNext;
     }
 
-    // if no A record found, then get address the hard way
+    // if no A or AAAA record found, then get address the hard way
     if (aRecord == NULL)
       PIPSocket::GetHostAddress(record->hostName, record->hostAddress);
   }
@@ -477,8 +519,12 @@ PBoolean PDNS::LookupSRV(
     user = user + "@";
 
   PIPSocketAddressAndPortVector::const_iterator r;
-  for (r = info.begin(); r != info.end(); ++r) 
-    returnList.AppendString(user + r->AsString(':'));
+  for (r = info.begin(); r != info.end(); ++r) {
+    if (r->GetAddress().GetVersion() == 6)
+      returnList.AppendString(user + "[" + r->GetAddress().AsString() + "]:" + PString(r->GetPort()));
+    else
+      returnList.AppendString(user + r->AsString(':'));
+  }
 
   return returnList.GetSize() != 0;;
 }
@@ -495,13 +541,12 @@ PBoolean PDNS::LookupSRV(
     return PFalse;
   }
 
-  PTRACE(4,"DNS\tSRV Lookup " << domain << " service " << service);
-  
   PString srvLookupStr = service;
   if (srvLookupStr.Right(1) != ".")
     srvLookupStr += ".";
   srvLookupStr += domain;
   
+  PTRACE(4,"DNS\tSRV Lookup \"" << srvLookupStr << '"');
   return LookupSRV(srvLookupStr, defaultPort, addrList);
 }
 
@@ -518,7 +563,8 @@ PBoolean PDNS::LookupSRV(
     PTRACE(5,"DNS\tSRV Record found " << srvLookupStr);
     PDNS::SRVRecord * recPtr = srvRecords.GetFirst();
     while (recPtr != NULL) {
-      PIPSocketAddressAndPort addrAndPort(recPtr->hostAddress, recPtr->port > 0 ? recPtr->port : defaultPort);
+      PIPSocketAddressAndPort addrAndPort;
+      addrAndPort.SetAddress(recPtr->hostAddress, recPtr->port > 0 ? recPtr->port : defaultPort);
       addrList.push_back(addrAndPort);
 
       recPtr = srvRecords.GetNext();
@@ -576,6 +622,10 @@ PDNS::MXRecord * PDNS::MXRecordList::HandleDNSRecord(PDNS_RECORD dnsRecord, PDNS
         record->hostAddress = PIPSocket::Address(dnsRecord->Data.A.IpAddress);
         break;
       }
+      if ((dnsRecord->Flags.S.Section == DnsSectionAdditional) && (dnsRecord->wType == DNS_TYPE_AAAA)) {
+        record->hostAddress = PIPSocket::Address(16, (BYTE *)&dnsRecord->Data.AAAA.Ip6Address);
+        break;
+      }
       aRecord = aRecord->pNext;
     }
 
@@ -616,6 +666,122 @@ PDNS::MXRecord * PDNS::MXRecordList::GetNext()
   return (PDNS::MXRecord *)GetAt(lastIndex++);
 }
 
+/////////////////////////////////////////////////////////////////
+
+#ifndef USE_RESOLVER_CACHING
+
+DNS_STATUS PDNS::Cached_DnsQuery(
+    const char * name,
+    WORD       type,
+    DWORD      ,
+    void *     ,
+    PDNS_RECORD * queryResults,
+    void * )
+{
+  return DnsQuery_A((const char *)name, 
+                               type,
+                               DNS_QUERY_STANDARD, 
+                               (PIP4_ARRAY)NULL, 
+                               queryResults, 
+                               NULL);
+}
+
+void PDNS::Cached_DnsRecordListFree(PDNS_RECORD results, int)
+{
+  DnsRecordListFree(results, 0);
+}
+
+#else  // ifndef USE_RESOLVER_CACHING
+
+static PDNS_RECORD DnsRecordListClone(PDNS_RECORD src)
+{
+  PDNS_RECORD result = NULL;
+  PDNS_RECORD dst = NULL;
+  PDNS_RECORD rec;
+
+  while (src != NULL) {
+    rec = (PDNS_RECORD)malloc(sizeof(DNS_RECORD));
+    memcpy(rec, src, sizeof(DNS_RECORD));
+    if (result == NULL)
+      result = rec;
+    rec->pNext = NULL;
+    if (dst != NULL)
+      dst->pNext = rec;
+    src = src->pNext;
+    dst = rec;
+  }
+
+  return result;
+}
+
+DNS_STATUS PDNS::Cached_DnsQuery(
+    const char * name,
+    WORD       type,
+    DWORD      options,
+    void *     ,
+    PDNS_RECORD * queryResults,
+    void * )
+{
+  PTime now;
+  PWaitAndSignal m(GetDNSMutex());
+
+  DNSCache::iterator r;
+
+  // age entries in cache
+  if ((now - g_lastAgeTime) > RESOLVER_CACHE_TIMEOUT) {
+    g_lastAgeTime = now;
+
+    r = g_dnsCache.begin();
+    while (r != g_dnsCache.end()) {
+      if ((now - r->second.m_time) < RESOLVER_CACHE_TIMEOUT)
+        ++r;
+      else { 
+        DnsRecordListFree(r->second.m_results, DnsFreeFlat);
+        g_dnsCache.erase(r++);
+      }
+    }
+  }
+
+  // see if cache contains the entry we need
+  std::stringstream key;
+  key << name << '\n' << type << '\n' << options;
+
+  r = g_dnsCache.find(key.str());
+  if (r == g_dnsCache.end()) {
+
+    // else do the lookup and put it into the cache
+    DNSCacheInfo info;
+    info.m_status = DnsQuery_A((const char *)name, 
+                               type,
+                               DNS_QUERY_STANDARD, 
+                               (PIP4_ARRAY)NULL, 
+                               &info.m_results, 
+                               NULL);
+    PTRACE_IF(3, info.m_status != 0, "DNS\tSRV query failed, error=" << info.m_status);
+
+    r = g_dnsCache.insert(DNSCache::value_type(key.str(), info)).first;
+  }
+
+  *queryResults = DnsRecordListClone(r->second.m_results);
+  return r->second.m_status;
+}
+
+void PDNS::Cached_DnsRecordListFree(PDNS_RECORD results, int)
+{
+  PDnsRecordListFree(results, 0);
+}
+
+
+#endif  // ifndef USE_RESOLVER_CACHING
+
+/////////////////////////////////////////////////////////////////
+
+#else
+
+  #ifdef _MSC_VER
+    #pragma message("DNS support DISABLED")
+  #endif
+
 #endif // P_DNS
 
 
