@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24000 $
+ * $Revision: 28201 $
  * $Author: rjongbloed $
- * $Date: 2010-01-31 17:57:16 -0600 (Sun, 31 Jan 2010) $
+ * $Date: 2012-08-14 21:30:31 -0500 (Tue, 14 Aug 2012) $
  */
 
 #define _OSUTIL_CXX
@@ -48,11 +48,11 @@
 #pragma implementation "object.h"
 #pragma implementation "contain.h"
 
-#if defined(P_LINUX)
+#if defined(P_LINUX) || defined(P_GNU_HURD)
 #ifndef _REENTRANT
 #define _REENTRANT
 #endif
-#elif defined(P_SOLARIS) 
+#elif defined(P_SOLARIS)
 #define _POSIX_PTHREAD_SEMANTICS
 #endif
 
@@ -65,10 +65,11 @@
 #else
 #include <time.h>
 #include <sys/time.h>
+#include <termios.h>
 #endif
 #include <ctype.h>
 
-#if defined(P_LINUX)
+#if defined(P_LINUX) || defined(P_GNU_HURD)
 
 #include <mntent.h>
 #include <sys/vfs.h>
@@ -84,6 +85,7 @@
 
 #include <sys/param.h>
 #include <sys/mount.h>
+#include <sys/timeb.h>
 
 #if defined(P_NETBSD)
 #include <sys/statvfs.h>
@@ -98,7 +100,6 @@
 
 #include <fstab.h>
 #include <sys/stat.h>
-#include <sys/statfs.h>
 
 #elif defined(P_SOLARIS) 
 #define P_USE_LANGINFO
@@ -506,7 +507,7 @@ PString PDirectory::GetVolume() const
   if (stat(operator+("."), &status) != -1) {
     dev_t my_dev = status.st_dev;
 
-#if defined(P_LINUX) || defined(P_IRIX)
+#if defined(P_LINUX) || defined(P_IRIX) || defined(P_GNU_HURD)
 
     FILE * fp = setmntent(MOUNTED, "r");
     if (fp != NULL) {
@@ -574,7 +575,7 @@ PString PDirectory::GetVolume() const
 
 PBoolean PDirectory::GetVolumeSpace(PInt64 & total, PInt64 & free, DWORD & clusterSize) const
 {
-#if defined(P_LINUX) || defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS)
+#if defined(P_LINUX) || defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS) || defined(P_GNU_HURD)
 
   struct statfs fs;
 
@@ -1156,12 +1157,34 @@ PBoolean PConsoleChannel::Close()
 }
 
 
+bool PConsoleChannel::SetLocalEcho(bool localEcho)
+{
+  if (!IsOpen())
+    return ConvertOSError(-2, LastReadError);
+
+#ifdef P_VXWORKS
+  PAssertAlways("PConsoleChannel::GetName - Not implemented for VxWorks");
+  return PString("Not Implemented");
+#else
+  struct termios ios;
+  if (!ConvertOSError(tcgetattr(os_handle, &ios)))
+    return false;
+
+  if (localEcho)
+    ios.c_lflag |= ECHO;
+  else
+    ios.c_lflag &= ~ECHO;
+  return ConvertOSError(tcsetattr(os_handle, TCSANOW, &ios));
+#endif
+}
+
+
 //////////////////////////////////////////////////////
 //
 //  PTime
 //
 
-PTime::PTime()
+void PTime::SetCurrentTime()
 {
 #ifdef P_VXWORKS
   struct timespec ts;
@@ -1239,10 +1262,10 @@ PString PTime::GetTimePM()
 
 PString PTime::GetTimeSeparator()
 {
-#if defined(P_LINUX) || defined(P_HPUX9) || defined(P_SOLARIS) || defined(P_IRIX)
+#if defined(P_LINUX) || defined(P_HPUX9) || defined(P_SOLARIS) || defined(P_IRIX) || defined(P_GNU_HURD)
 #  if defined(P_USE_LANGINFO)
      char * p = nl_langinfo(T_FMT);
-#  elif defined(P_LINUX)
+#  elif defined(P_LINUX) || defined(P_GNU_HURD)
      char * p = _time_info->time; 
 #  endif
   char buffer[2];
@@ -1272,7 +1295,7 @@ PString PTime::GetTimeSeparator()
 
 PTime::DateOrder PTime::GetDateOrder()
 {
-#if defined(P_USE_LANGINFO) || defined(P_LINUX)
+#if defined(P_USE_LANGINFO) || defined(P_LINUX) || defined(P_GNU_HURD)
 #  if defined(P_USE_LANGINFO)
      char * p = nl_langinfo(D_FMT);
 #  else
@@ -1316,7 +1339,7 @@ PTime::DateOrder PTime::GetDateOrder()
 
 PString PTime::GetDateSeparator()
 {
-#if defined(P_USE_LANGINFO) || defined(P_LINUX)
+#if defined(P_USE_LANGINFO) || defined(P_LINUX) || defined(P_GNU_HURD)
 #  if defined(P_USE_LANGINFO)
      char * p = nl_langinfo(D_FMT);
 #  else
@@ -1355,7 +1378,7 @@ PString PTime::GetDayName(PTime::Weekdays day, NameType type)
                    nl_langinfo((nl_item)(DAY_1+(int)day))
                 );
 
-#elif defined(P_LINUX)
+#elif defined(P_LINUX) || defined(P_GNU_HURD)
   return (type == Abbreviated) ? PString(_time_info->abbrev_wkday[(int)day]) :
                        PString(_time_info->full_wkday[(int)day]);
 
@@ -1388,7 +1411,7 @@ PString PTime::GetMonthName(PTime::Months month, NameType type)
      (type == Abbreviated) ? nl_langinfo((nl_item)(ABMON_1+(int)month-1)) :
                    nl_langinfo((nl_item)(MON_1+(int)month-1))
                 );
-#elif defined(P_LINUX)
+#elif defined(P_LINUX) || defined(P_GNU_HURD)
   return (type == Abbreviated) ? PString(_time_info->abbrev_month[(int)month-1]) :
                        PString(_time_info->full_month[(int)month-1]);
 #elif defined(P_USE_STRFTIME)
@@ -1423,13 +1446,13 @@ PBoolean PTime::IsDaylightSavings()
 
 int PTime::GetTimeZone(PTime::TimeZoneType type) 
 {
-#if defined(P_LINUX) || defined(P_SOLARIS) || defined (P_AIX) || defined(P_IRIX)
+#if defined(P_LINUX) || defined(P_SOLARIS) || defined (P_AIX) || defined(P_IRIX) || defined(P_GNU_HURD)
   long tz = -::timezone/60;
   if (type == StandardTime)
     return tz;
   else
     return tz + ::daylight*60;
-#elif defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS) || defined(__BEOS__) || defined(P_QNX)
+#elif defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS) || defined(__BEOS__) || defined(P_QNX) || defined(P_GNU_HURD)
   time_t t;
   time(&t);
   struct tm ts;
